@@ -4,7 +4,7 @@
 package Net::Frame::Layer::NTP;
 use strict; use warnings;
 
-our $VERSION = '1.01';
+our $VERSION = '1.02';
 
 use Net::Frame::Layer qw(:consts :subs);
 use Exporter;
@@ -27,6 +27,40 @@ our %EXPORT_TAGS = (
       NF_NTP_MODE_PRIVATE
       NF_NTP_STRATUM_UNSPEC
       NF_NTP_STRATUM_PRIMARY
+      NF_NTP_STRATUM_UNSYNC
+      NF_NTP_REFID_GOES
+      NF_NTP_REFID_GPS
+      NF_NTP_REFID_GAL
+      NF_NTP_REFID_PPS
+      NF_NTP_REFID_IRIG
+      NF_NTP_REFID_WWVB
+      NF_NTP_REFID_DCF
+      NF_NTP_REFID_HBG
+      NF_NTP_REFID_MSF
+      NF_NTP_REFID_JJY
+      NF_NTP_REFID_LORC
+      NF_NTP_REFID_TDF
+      NF_NTP_REFID_CHU
+      NF_NTP_REFID_WWV
+      NF_NTP_REFID_WWVH
+      NF_NTP_REFID_NIST
+      NF_NTP_REFID_ACTS
+      NF_NTP_REFID_USNO
+      NF_NTP_REFID_PTB
+      NF_NTP_KoD_ACST
+      NF_NTP_KoD_AUTH
+      NF_NTP_KoD_AUTO
+      NF_NTP_KoD_BCST
+      NF_NTP_KoD_CRYP
+      NF_NTP_KoD_DENY
+      NF_NTP_KoD_DROP
+      NF_NTP_KoD_RSTR
+      NF_NTP_KoD_INIT
+      NF_NTP_KoD_MCST
+      NF_NTP_KoD_NKEY
+      NF_NTP_KoD_RATE
+      NF_NTP_KoD_RMOT
+      NF_NTP_KoD_STEP
    )],
    subs => [qw(
       ntpTimestamp
@@ -53,25 +87,59 @@ use constant NF_NTP_MODE_NTPCONTROL => 6;
 use constant NF_NTP_MODE_PRIVATE    => 7;
 use constant NF_NTP_STRATUM_UNSPEC  => 0;
 use constant NF_NTP_STRATUM_PRIMARY => 1;
+use constant NF_NTP_STRATUM_UNSYNC  => 16;
+use constant NF_NTP_REFID_GOES      => 0x474f4553;
+use constant NF_NTP_REFID_GPS       => 0x47505300;
+use constant NF_NTP_REFID_GAL       => 0x47414c00;
+use constant NF_NTP_REFID_PPS       => 0x50505300;
+use constant NF_NTP_REFID_IRIG      => 0x49524947;
+use constant NF_NTP_REFID_WWVB      => 0x57575642;
+use constant NF_NTP_REFID_DCF       => 0x44434600;
+use constant NF_NTP_REFID_HBG       => 0x48424700;
+use constant NF_NTP_REFID_MSF       => 0x4d534600;
+use constant NF_NTP_REFID_JJY       => 0x4a4a5900;
+use constant NF_NTP_REFID_LORC      => 0x4c4f5243;
+use constant NF_NTP_REFID_TDF       => 0x54444600;
+use constant NF_NTP_REFID_CHU       => 0x43485500;
+use constant NF_NTP_REFID_WWV       => 0x57575600;
+use constant NF_NTP_REFID_WWVH      => 0x57575648;
+use constant NF_NTP_REFID_NIST      => 0x4e495354;
+use constant NF_NTP_REFID_ACTS      => 0x41435453;
+use constant NF_NTP_REFID_USNO      => 0x55534e4f;
+use constant NF_NTP_REFID_PTB       => 0x50544200;
+use constant NF_NTP_KoD_ACST        => 0x41435354;
+use constant NF_NTP_KoD_AUTH        => 0x41555448;
+use constant NF_NTP_KoD_AUTO        => 0x4155544f;
+use constant NF_NTP_KoD_BCST        => 0x42435354;
+use constant NF_NTP_KoD_CRYP        => 0x43525950;
+use constant NF_NTP_KoD_DENY        => 0x44454e59;
+use constant NF_NTP_KoD_DROP        => 0x44524f50;
+use constant NF_NTP_KoD_RSTR        => 0x52535452;
+use constant NF_NTP_KoD_INIT        => 0x494e4954;
+use constant NF_NTP_KoD_MCST        => 0x4d435354;
+use constant NF_NTP_KoD_NKEY        => 0x4e4b4559;
+use constant NF_NTP_KoD_RATE        => 0x52415445;
+use constant NF_NTP_KoD_RMOT        => 0x524d4f54;
+use constant NF_NTP_KoD_STEP        => 0x53544550;
 
 our @AS = qw(
-   li
+   leap
    version
    mode
    stratum
-   pollInterval
+   poll
    precision
    rootDelay
-   rootDispersion
-   refClockId
-   refTimestamp
-   refTimestamp_frac
-   origTimestamp
-   origTimestamp_frac
-   recvTimestamp
-   recvTimestamp_frac
-   xmitTimestamp
-   xmitTimestamp_frac
+   rootDisp
+   refId
+   refTime
+   refTime_frac
+   org
+   org_frac
+   rec
+   rec_frac
+   xmt
+   xmt_frac
 );
 __PACKAGE__->cgBuildIndices;
 __PACKAGE__->cgBuildAccessorsScalar(\@AS);
@@ -86,23 +154,23 @@ $Net::Frame::Layer::UDP::Next->{123} = "NTP";
 sub new {
 
    shift->SUPER::new(
-      li             => NF_NTP_LI_NOWARN,
-      version        => 3,
-      mode           => NF_NTP_MODE_CLIENT,
-      stratum        => NF_NTP_STRATUM_UNSPEC,
-      pollInterval   => 0,
-      precision      => 0,
-      rootDelay      => 0,
-      rootDispersion => 0,
-      refClockId     => 0,
-      refTimestamp   => 0,
-      refTimestamp_frac  => 0,
-      origTimestamp      => 0,
-      origTimestamp_frac => 0,
-      recvTimestamp      => 0,
-      recvTimestamp_frac => 0,
-      xmitTimestamp      => ntpTimestamp(time),
-      xmitTimestamp_frac => 0,
+      leap         => NF_NTP_LI_NOWARN,
+      version      => 3,
+      mode         => NF_NTP_MODE_CLIENT,
+      stratum      => NF_NTP_STRATUM_UNSPEC,
+      poll         => 0,
+      precision    => 0,
+      rootDelay    => 0,
+      rootDisp     => 0,
+      refId        => 0,
+      refTime      => 0,
+      refTime_frac => 0,
+      org          => 0,
+      org_frac     => 0,
+      rec          => 0,
+      rec_frac     => 0,
+      xmt          => ntpTimestamp(time),
+      xmt_frac     => 0,
       @_,
    );
 }
@@ -112,27 +180,27 @@ sub getLength { 48 }
 sub pack {
    my $self = shift;
 
-   my $li      = Bit::Vector->new_Dec(2, $self->li);
+   my $leap    = Bit::Vector->new_Dec(2, $self->leap);
    my $version = Bit::Vector->new_Dec(3, $self->version);
    my $mode    = Bit::Vector->new_Dec(3, $self->mode);
-   my $bvlist = $li->Concat_List($version, $mode);
+   my $bvlist  = $leap->Concat_List($version, $mode);
 
    my $raw = $self->SUPER::pack('CCCC N11',
       $bvlist->to_Dec,
       $self->stratum,
-      $self->pollInterval,
+      $self->poll,
       $self->precision,
       $self->rootDelay,
-      $self->rootDispersion,
-      $self->refClockId,
-      $self->refTimestamp,
-      $self->refTimestamp_frac,
-      $self->origTimestamp,
-      $self->origTimestamp_frac,
-      $self->recvTimestamp,
-      $self->recvTimestamp_frac,
-      $self->xmitTimestamp,
-      $self->xmitTimestamp_frac,
+      $self->rootDisp,
+      $self->refId,
+      $self->refTime,
+      $self->refTime_frac,
+      $self->org,
+      $self->org_frac,
+      $self->rec,
+      $self->rec_frac,
+      $self->xmt,
+      $self->xmt_frac,
    ) or return;
 
    return $self->raw($raw);
@@ -141,36 +209,36 @@ sub pack {
 sub unpack {
    my $self = shift;
 
-   my ($bv, $stratum, $pollInterval, $precision,
-       $rootDelay, $rootDispersion, 
-       $refClockId,
-       $refTimestamp, $refTimestamp_frac,
-       $origTimestamp, $origTimestamp_frac,
-       $recvTimestamp, $recvTimestamp_frac,
-       $xmitTimestamp, $xmitTimestamp_frac,
+   my ($bv, $stratum, $poll, $precision,
+       $rootDelay, $rootDisp, 
+       $refId,
+       $refTime, $refTime_frac,
+       $org, $org_frac,
+       $rec, $rec_frac,
+       $xmt, $xmt_frac,
        $payload) =
       $self->SUPER::unpack('CCCC N N H8 N8 a*', $self->raw)
          or return;
 
    my $bvlist = Bit::Vector->new_Dec(8, $bv);
-   $self->li     ($bvlist->Chunk_Read(2,6));
+   $self->leap   ($bvlist->Chunk_Read(2,6));
    $self->version($bvlist->Chunk_Read(3,3));
    $self->mode   ($bvlist->Chunk_Read(3,0));
 
    $self->stratum($stratum);
-   $self->pollInterval($pollInterval);
+   $self->poll($poll);
    $self->precision($precision);
    $self->rootDelay($rootDelay);
-   $self->rootDispersion($rootDispersion);
-   $self->refClockId(_unpack_refid($stratum, $refClockId));
-   $self->refTimestamp($refTimestamp);
-   $self->refTimestamp_frac($refTimestamp_frac);
-   $self->origTimestamp($origTimestamp);
-   $self->origTimestamp_frac($origTimestamp_frac);
-   $self->recvTimestamp($recvTimestamp);
-   $self->recvTimestamp_frac($recvTimestamp_frac);
-   $self->xmitTimestamp($xmitTimestamp);
-   $self->xmitTimestamp_frac($xmitTimestamp_frac);
+   $self->rootDisp($rootDisp);
+   $self->refId(_unpack_refid($stratum, $refId));
+   $self->refTime($refTime);
+   $self->refTime_frac($refTime_frac);
+   $self->org($org);
+   $self->org_frac($org_frac);
+   $self->rec($rec);
+   $self->rec_frac($rec_frac);
+   $self->xmt($xmt);
+   $self->xmt_frac($xmt_frac);
 
    $self->payload($payload);
 
@@ -193,35 +261,35 @@ sub encapsulate {
 sub print {
    my $self = shift;
 
-   my $refTimestamp_frac = _bin2frac(_dec2bin($self->refTimestamp_frac));
-   my $origTimestamp_frac = _bin2frac(_dec2bin($self->origTimestamp_frac));
-   my $recvTimestamp_frac = _bin2frac(_dec2bin($self->recvTimestamp_frac));
-   my $xmitTimestamp_frac = _bin2frac(_dec2bin($self->xmitTimestamp_frac));
+   my $refTime_frac = _bin2frac(_dec2bin($self->refTime_frac));
+   my $org_frac = _bin2frac(_dec2bin($self->org_frac));
+   my $rec_frac = _bin2frac(_dec2bin($self->rec_frac));
+   my $xmt_frac = _bin2frac(_dec2bin($self->xmt_frac));
 
    my $l = $self->layer;
    my $buf = sprintf
-      "$l: li:%d  version:%d  mode:%d  stratum:%d\n".
-      "$l: pollInterval:%d  precision:%d\n".
-      "$l: rootDelay:%d  rootDispersion:%d  refClockId:%s\n".
-      "$l: refTimestamp:%d   refTimestamp_frac:%s\n".
+      "$l: leap:%d  version:%d  mode:%d  stratum:%d\n".
+      "$l: poll:%d  precision:%d\n".
+      "$l: rootDelay:%d  rootDisp:%d  refId:%s\n".
+      "$l: refTime:%d   refTime_frac:%s\n".
 #      "$l:   [%s%s]\n".
-      "$l: origTimestamp:%d  origTimestamp_frac:%s\n".
+      "$l: org:%d  org_frac:%s\n".
 #      "$l:   [%s%s]\n".
-      "$l: recvTimestamp:%d  recvTimestamp_frac:%s\n".
+      "$l: rec:%d  rec_frac:%s\n".
 #      "$l:   [%s%s]\n".
-      "$l: xmitTimestamp:%d  xmitTimestamp_frac:%s\n",
+      "$l: xmt:%d  xmt_frac:%s\n",
 #      "$l:   [%s%s]",
-         $self->li, $self->version, $self->mode, $self->stratum,
-         $self->pollInterval, $self->precision,
-         $self->rootDelay, $self->rootDispersion, $self->refClockId,
-         $self->refTimestamp, $self->refTimestamp_frac,
-#         _getTime($self->refTimestamp  + $refTimestamp_frac  - NF_NTP_ADJ), substr($refTimestamp_frac, 1),
-         $self->origTimestamp, $self->origTimestamp_frac, 
-#         _getTime($self->origTimestamp + $origTimestamp_frac - NF_NTP_ADJ), substr($origTimestamp_frac, 1),
-         $self->recvTimestamp, $self->recvTimestamp_frac, 
-#         _getTime($self->recvTimestamp + $recvTimestamp_frac - NF_NTP_ADJ), substr($recvTimestamp_frac, 1),
-         $self->xmitTimestamp, $self->xmitTimestamp_frac;
-#         _getTime($self->xmitTimestamp + $xmitTimestamp_frac - NF_NTP_ADJ), substr($xmitTimestamp_frac, 1);
+         $self->leap, $self->version, $self->mode, $self->stratum,
+         $self->poll, $self->precision,
+         $self->rootDelay, $self->rootDisp, $self->refId,
+         $self->refTime, $self->refTime_frac,
+#         _getTime($self->refTime + $refTime_frac - NF_NTP_ADJ), substr($refTime_frac, 1),
+         $self->org, $self->org_frac, 
+#         _getTime($self->org + $org_frac - NF_NTP_ADJ), substr($org_frac, 1),
+         $self->rec, $self->rec_frac, 
+#         _getTime($self->rec + $rec_frac - NF_NTP_ADJ), substr($rec_frac, 1),
+         $self->xmt, $self->xmt_frac;
+#         _getTime($self->xmt + $xmt_frac - NF_NTP_ADJ), substr($xmt_frac, 1);
 
    return $buf;
 }
@@ -300,23 +368,23 @@ Net::Frame::Layer::NTP - NTP layer object
    use Net::Frame::Layer::NTP qw(:consts);
 
    my $layer = Net::Frame::Layer::NTP->new(
-      li             => NF_NTP_LI_NOWARN,
-      version        => 3,
-      mode           => NF_NTP_MODE_CLIENT,
-      stratum        => NF_NTP_STRATUM_UNSPEC,
-      pollInterval   => 0,
-      precision      => 0,
-      rootDelay      => 0,
-      rootDispersion => 0,
-      refClockId     => 0,
-      refTimestamp   => 0,
-      refTimestamp_frac  => 0,
-      origTimestamp      => 0,
-      origTimestamp_frac => 0,
-      recvTimestamp      => 0,
-      recvTimestamp_frac => 0,
-      xmitTimestamp      => ntpTimestamp(time),
-      xmitTimestamp_frac => 0,
+      leap         => NF_NTP_LI_NOWARN,
+      version      => 3,
+      mode         => NF_NTP_MODE_CLIENT,
+      stratum      => NF_NTP_STRATUM_UNSPEC,
+      poll         => 0,
+      precision    => 0,
+      rootDelay    => 0,
+      rootDisp     => 0,
+      refId        => 0,
+      refTime      => 0,
+      refTime_frac => 0,
+      org          => 0,
+      org_frac     => 0,
+      rec          => 0,
+      rec_frac     => 0,
+      xmt          => ntpTimestamp(time),
+      xmt_frac     => 0,
    );
 
    #
@@ -341,7 +409,7 @@ See also B<Net::Frame::Layer> for other attributes and methods.
 
 =over 4
 
-=item B<li>
+=item B<leap>
 
 NTP Leap Indicator.  See B<CONSTANTS> for more information.
 
@@ -357,7 +425,7 @@ NTP mode.  See B<CONSTANTS> for more information.
 
 NTP stratum.  See B<CONSTANTS> for more information.
 
-=item B<pollInterval>
+=item B<poll>
 
 Maximum poll interval between messages in seconds to the nearest power of two.
 
@@ -369,35 +437,35 @@ Precision of the local clock in seconds to the nearest power of two.
 
 Total roundtrip delay to the primary reference source, in seconds with the fraction point between bits 15 and 16.
 
-=item B<rootDispersion>
+=item B<rootDisp>
 
 Maximum error relative to the primary reference source in seconds with the fraction point between bits 15 and 16.
 
-=item B<refClockId>
+=item B<refId>
 
 In the case of stratum 2 or greater, this is the IPv4 address of the primary reference host.  In the case of stratum 0 or 1, this is a four byte, left-justified, zero padded ASCII string.
 
-=item B<refTimestamp>
+=item B<ref>
 
-=item B<refTimestamp_frac>
+=item B<ref_frac>
 
 The local time at which the local clock was last set or corrected and the fractional part.
 
-=item B<origTimestamp>
+=item B<org>
 
-=item B<origTimestamp_frac>
+=item B<org_frac>
 
 The local time when the client sent the request and the fractional part.
 
-=item B<recvTimestamp>
+=item B<rec>
 
-=item B<recvTimestamp_frac>
+=item B<rec_frac>
 
 The local time when the request was received by the server and the fractional part.
 
-=item B<xmitTimestamp>
+=item B<xmt>
 
-=item B<xmitTimestamp_frac>
+=item B<xmt_frac>
 
 The local time when the reply was sent from the server and the fractional part.
 
@@ -509,8 +577,80 @@ NTP modes.
 
 =item B<NF_NTP_STRATUM_PRIMARY>
 
+=item B<NF_NTP_STRATUM_UNSYNC>
+
 NTP stratums.
 
+=item B<NF_NTP_REFID_GOES>
+
+=item B<NF_NTP_REFID_GPS>
+
+=item B<NF_NTP_REFID_GAL>
+
+=item B<NF_NTP_REFID_PPS>
+
+=item B<NF_NTP_REFID_IRIG>
+
+=item B<NF_NTP_REFID_WWVB>
+
+=item B<NF_NTP_REFID_DCF>
+
+=item B<NF_NTP_REFID_HBG>
+
+=item B<NF_NTP_REFID_MSF>
+
+=item B<NF_NTP_REFID_JJY>
+
+=item B<NF_NTP_REFID_LORC>
+
+=item B<NF_NTP_REFID_TDF>
+
+=item B<NF_NTP_REFID_CHU>
+
+=item B<NF_NTP_REFID_WWV>
+
+=item B<NF_NTP_REFID_WWVH>
+
+=item B<NF_NTP_REFID_NIST>
+
+=item B<NF_NTP_REFID_ACTS>
+
+=item B<NF_NTP_REFID_USNO>
+
+=item B<NF_NTP_REFID_PTB>
+
+NTP reference ID codes.
+
+=item B<NF_NTP_KoD_ACST>
+
+=item B<NF_NTP_KoD_AUTH>
+
+=item B<NF_NTP_KoD_AUTO>
+
+=item B<NF_NTP_KoD_BCST>
+
+=item B<NF_NTP_KoD_CRYP>
+
+=item B<NF_NTP_KoD_DENY>
+
+=item B<NF_NTP_KoD_DROP>
+
+=item B<NF_NTP_KoD_RSTR>
+
+=item B<NF_NTP_KoD_INIT>
+
+=item B<NF_NTP_KoD_MCST>
+
+=item B<NF_NTP_KoD_NKEY>
+
+=item B<NF_NTP_KoD_RATE>
+
+=item B<NF_NTP_KoD_RMOT>
+
+=item B<NF_NTP_KoD_STEP>
+
+NTP kiss codes.
+
 =back
 
 =head1 SEE ALSO
